/******************************************************************************
*
* project name:    ExtGraph
* initial date:    12/04/2001
* author:          thomas.nussbaumer@gmx.net
* description:     extgraph library header file
*
* $Id: extgraph.h,v 1.2 2001/06/22 14:33:01 Thomas Nussbaumer Exp $
*
*******************************************************************************/
#if !defined(__EXTGRAPH__)
#define __EXTGRAPH__

#include <gray.h>   // necessary for GetPlane() macros in the grayscale support
                    // routines !!

#define EXTGRAPH_VERSION_MAIN 0
#define EXTGRAPH_VERSION_SUB  86

enum GrayColors {COLOR_WHITE=1,COLOR_LIGHTGRAY=2,COLOR_DARKGRAY=4,COLOR_BLACK=8};
enum FillAttrs  {RECT_EMPTY=0,RECT_FILLED};
enum ExtAttrs   {A_CENTERED=0x40,A_SHADOWED=0x80};


/*---------------------------------------------------------------------------*/
/* speedy pixel access routines                                              */
/*---------------------------------------------------------------------------*/
#define EXT_PIXOFFSET(x,y)  ((y<<5)-(y<<1)+(x>>3))
#define EXT_PIXADDR(p,x,y)  (((unsigned char*)(p))+EXT_PIXOFFSET(x,y))
#define EXT_PIXMASK(x)      ((unsigned char)(0x80 >> ((x)&7)))

#define EXT_SETPIX_AM(a,m)   (*(a) |= (m))
#define EXT_CLRPIX_AM(a,m)   (*(a) &= ~(m))
#define EXT_XORPIX_AM(a,m)   (*(a) ^= (m))
#define EXT_GETPIX_AM(a,m)   (*(a) & (m))

#define EXT_PIXUP_AM(a)      ((a)-=30)
#define EXT_PIXDOWN_AM(a)    ((a)+=30)
#define EXT_PIXLEFT_AM(a,m)   asm("rol.b  #1,%0;bcc.s  0f;subq.l #1,%1;0:"\
                                  : "=d" (m), "=g" (a) : "0" (m), "1" (a))
#define EXT_PIXRIGHT_AM(a,m)  asm("ror.b  #1,%0;bcc.s  0f;addq.l #1,%1;0:"\
                                  : "=d" (m), "=g" (a) : "0" (m), "1" (a))

#define EXT_SETPIX(p,x,y)   EXT_SETPIX_AM(EXT_PIXADDR(p,x,y),EXT_PIXMASK(x))
#define EXT_CLRPIX(p,x,y)   EXT_CLRPIX_AM(EXT_PIXADDR(p,x,y),EXT_PIXMASK(x))
#define EXT_XORPIX(p,x,y)   EXT_XORPIX_AM(EXT_PIXADDR(p,x,y),EXT_PIXMASK(x))
#define EXT_GETPIX(p,x,y)   EXT_GETPIX_AM(EXT_PIXADDR(p,x,y),EXT_PIXMASK(x))

//-----------------------------------------------------------------------------
// double buffer variants of grayscale support routines
// NOTE: graph functions modifies active plane with PortSet() !!
//-----------------------------------------------------------------------------
extern void ClearGrayScreen2B(void* lightplane,void* darkplane);
extern void DrawGrayRect2B(short x0,short y0,short x1, short y1, short color,short fill,void* lightplane,void* darkplane);
extern void InvertGrayRect2B(short x0,short y0,short x1, short y1,void* lightplane,void* darkplane);
extern void DrawGrayLine2B(short x0,short y0,short x1, short y1, short color,void* lightplane,void* darkplane);
extern void FastDrawGrayLine2B(short x0,short y0,short x1, short y1, short color,void* lightplane,void* darkplane);
extern void FastDrawGrayHLine2B(short x0,short x1, short y, short color,void* lightplane,void* darkplane);
extern void DrawGrayChar2B(short x,short y,char c,short attr,void* lightplane,void* darkplane);
extern void DrawGrayStr2B(short x,short y,char* s,short attr,void* lightplane,void* darkplane);
extern void DrawGrayStrExt2B(short x,short y,char* s,short attr,short font,void* lightplane,void* darkplane);

//-----------------------------------------------------------------------------
// NOTE: graph functions modifies active plane with PortSet() !!
//-----------------------------------------------------------------------------
#define ClearGrayScreen()                    ClearGrayScreen2B(GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))
#define DrawGrayRect(x0,y0,x1,y1,color,fill) DrawGrayRect2B(x0,y0,x1,y1,color,fill,GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))
#define InvertGrayRect(x0,y0,x1,y1)          InvertGrayRect2B(x0,y0,x1,y1,GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))
#define DrawGrayLine(x0,y0,x1,y1,color)      DrawGrayLine2B(x0,y0,x1,y1,color,GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))
#define FastDrawGrayHLine(x0,x1,y,color)     FastDrawGrayHLine2B(x0,x1,y,color,GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))
#define FastDrawGrayLine(x0,y0,x1,y1,color)  FastDrawGrayLine2B(x0,y0,x1,y1,color,GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))

#define DrawGrayChar(x,y,c,attr)             DrawGrayChar2B(x,y,c,attr,GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))
#define DrawGrayStr(x,y,s,attr)              DrawGrayStr2B(x,y,s,attr,GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))
#define DrawGrayStrExt(x,y,s,attr,font)      DrawGrayStrExt2B(x,y,s,attr,font,GetPlane(LIGHT_PLANE),GetPlane(DARK_PLANE))

//-----------------------------------------------------------------------------
// screen scrolling routines
//-----------------------------------------------------------------------------
extern void ScrollLeft160(unsigned short* buffer,unsigned short lines);
extern void ScrollLeft240(unsigned short* buffer,unsigned short lines);
extern void ScrollRight160(unsigned short* buffer,unsigned short lines);
extern void ScrollRight240(unsigned short* buffer,unsigned short lines);
extern void ScrollUp160(unsigned short* buffer,unsigned short lines);
extern void ScrollUp240(unsigned short* buffer,unsigned short lines);
extern void ScrollDown160(unsigned short* buffer,unsigned short lines);
extern void ScrollDown240(unsigned short* buffer,unsigned short lines);

//-----------------------------------------------------------------------------
// fast copy routine for a complete screen (240x128 pixels == 3840 bytes)
//-----------------------------------------------------------------------------
extern void FastCopyScreen(void* src,void* dest);

//-----------------------------------------------------------------------------
// fast alternative functions for line drawing
//-----------------------------------------------------------------------------
extern void FastDrawHLine(unsigned char* plane,short x1,short x2,short y,short mode);
extern void FastDrawLine(unsigned char* plane,short x1,short y1,short x2,short y2,short mode);

//-----------------------------------------------------------------------------
// fast alternative functions for SpriteX() functions
//-----------------------------------------------------------------------------
extern void  Sprite8_OR(short x,short y,short h,unsigned char* sprite,void* dest);
extern void Sprite16_OR(short x,short y,short h,unsigned short* sprite,void* dest);
extern void Sprite32_OR(short x,short y,short h,unsigned long* sprite,void* dest);
extern void SpriteX8_OR(short x,short y,short h,unsigned char* sprite,short bytewidth,void* dest);
extern void  GraySprite8_OR(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,void* dest1,void* dest2);
extern void GraySprite16_OR(short x,short y,short h,unsigned short* sprite1,unsigned short* sprite2,void* dest1,void* dest2);
extern void GraySprite32_OR(short x,short y,short h,unsigned long* sprite1,unsigned long* sprite2,void* dest1,void* dest2);
extern void GraySpriteX8_OR(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,short bytewidth,void* dest1,void* dest2);

extern void  Sprite8_AND(short x,short y,short h,unsigned char* sprite,void* dest);
extern void Sprite16_AND(short x,short y,short h,unsigned short* sprite,void* dest);
extern void Sprite32_AND(short x,short y,short h,unsigned long* sprite,void* dest);
extern void SpriteX8_AND(short x,short y,short h,unsigned char* sprite,short bytewidth,void* dest);
extern void  GraySprite8_AND(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,void* dest1,void* dest2);
extern void GraySprite16_AND(short x,short y,short h,unsigned short* sprite1,unsigned short* sprite2,void* dest1,void* dest2);
extern void GraySprite32_AND(short x,short y,short h,unsigned long* sprite1,unsigned long* sprite2,void* dest1,void* dest2);
extern void GraySpriteX8_AND(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,short bytewidth,void* dest1,void* dest2);

extern void  Sprite8_XOR(short x,short y,short h,unsigned char* sprite,void* dest);
extern void Sprite16_XOR(short x,short y,short h,unsigned short* sprite,void* dest);
extern void Sprite32_XOR(short x,short y,short h,unsigned long* sprite,void* dest);
extern void SpriteX8_XOR(short x,short y,short h,unsigned char* sprite,short bytewidth,void* dest);
extern void  GraySprite8_XOR(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,void* dest1,void* dest2);
extern void GraySprite16_XOR(short x,short y,short h,unsigned short* sprite1,unsigned short* sprite2,void* dest1,void* dest2);
extern void GraySprite32_XOR(short x,short y,short h,unsigned long* sprite1,unsigned long* sprite2,void* dest1,void* dest2);
extern void GraySpriteX8_XOR(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,short bytewidth,void* dest1,void* dest2);

//-----------------------------------------------------------------------------
// additionally sprite functions (MASK & BLIT)
//-----------------------------------------------------------------------------
extern void  Sprite8_MASK(short x,short y,short h,unsigned char* sprite,unsigned char* mask,void* dest);
extern void Sprite16_MASK(short x,short y,short h,unsigned short* sprite,unsigned short* mask,void* dest);
extern void Sprite32_MASK(short x,short y,short h,unsigned long* sprite,unsigned long* mask,void* dest);
extern void SpriteX8_MASK(short x,short y,short h,unsigned char* sprite,unsigned char* mask,short bytewidth,void* dest);
extern void  GraySprite8_MASK(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,unsigned char* mask1,unsigned char* mask2,void* dest1,void* dest2);
extern void GraySprite16_MASK(short x,short y,short h,unsigned short* sprite1,unsigned short* sprite2,unsigned short* mask1,unsigned short* mask2,void* dest1,void* dest2);
extern void GraySprite32_MASK(short x,short y,short h,unsigned long* sprite1,unsigned long* sprite2,unsigned long* mask1,unsigned long* mask2,void* dest1,void* dest2);
extern void GraySpriteX8_MASK(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,unsigned char* mask1,unsigned char* mask2,short bytewidth,void* dest1,void* dest2);

extern void  Sprite8_BLIT(short x,short y,short h,unsigned char* sprite,unsigned char maskval,void* dest);
extern void Sprite16_BLIT(short x,short y,short h,unsigned short* sprite,unsigned short maskval,void* dest);
extern void Sprite32_BLIT(short x,short y,short h,unsigned long* sprite,unsigned long maskval,void* dest);
extern void SpriteX8_BLIT(short x,short y,short h,unsigned char* sprite,unsigned char* maskval,short bytewidth,void* dest);
extern void  GraySprite8_BLIT(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,unsigned char maskval,void* dest1,void* dest2);
extern void GraySprite16_BLIT(short x,short y,short h,unsigned short* sprite1,unsigned short* sprite2,unsigned short maskval,void* dest1,void* dest2);
extern void GraySprite32_BLIT(short x,short y,short h,unsigned long* sprite1,unsigned long* sprite2,unsigned long maskval,void* dest1,void* dest2);
extern void GraySpriteX8_BLIT(short x,short y,short h,unsigned char* sprite1,unsigned char* sprite2,unsigned char* maskval,short bytewidth,void* dest1,void* dest2);

//-----------------------------------------------------------------------------
// fast alternative functions for BitmapGet() functionality (not compatible)
//-----------------------------------------------------------------------------
extern void  Sprite8Get(short x,short y,short h,void* src,unsigned char* dest);
extern void Sprite16Get(short x,short y,short h,void* src,unsigned short* dest);
extern void Sprite32Get(short x,short y,short h,void* src,unsigned long* dest);
extern void SpriteX8Get(short x,short y,short h,void* src,unsigned char* dest,short bytewidth);

//-----------------------------------------------------------------------------
// sprite scaling routines by Jim Haskell (jimhaskell@yahoo.com)
//
// scales a square input sprite to any extent (WARNING: no clipping is done!)
//
// [routines slightly modified to fit the needs]
//-----------------------------------------------------------------------------
extern void ScaleSprite8_OR(unsigned char *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite16_OR(unsigned short *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite32_OR(unsigned long *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite64_OR(unsigned long long *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);

extern void ScaleSprite8_AND(unsigned char *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite16_AND(unsigned short *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite32_AND(unsigned long *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite64_AND(unsigned long long *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);

extern void ScaleSprite8_XOR(unsigned char *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite16_XOR(unsigned short *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite32_XOR(unsigned long *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);
extern void ScaleSprite64_XOR(unsigned long long *sprite,unsigned char *dest,short x0,short y0,short sizex,short sizey);

//-----------------------------------------------------------------------------
// floodfill routines (courtesy of Zeljko Juric)
//-----------------------------------------------------------------------------
extern void FloodFill(short x,short y,unsigned short shade,void* tmpplane,void* dest);
extern void FloodFillMF(short x,short y,unsigned short shade,void* dest);


//-----------------------------------------------------------------------------
// sprite x8 mirror functions
//-----------------------------------------------------------------------------
extern void SpriteX8_MIRROR_H(short h,unsigned char* src,short bytewidth,unsigned char* dest);
extern void SpriteX8_MIRROR_V(short h,unsigned char* src,short bytewidth,unsigned char* dest);


#endif

//#############################################################################
// Revision History
//#############################################################################
//
// $Log: extgraph.h,v $
// Revision 1.2  2001/06/22 14:33:01  Thomas Nussbaumer
// grayscale sprite drawing routines added
//
// Revision 1.1  2001/06/20 21:34:08  Thomas Nussbaumer
// initial check-in
//
//
